@@ -1,4 +1,6 @@
 # -*- coding: utf-8 -*-
+import logging
+
 from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
 from django.core.urlresolvers import resolve
 from django.http import Http404
@@ -7,12 +9,16 @@ from django.utils.translation import (
     activate as translation_activate, check_for_language, ugettext as _
 )
 from django.views.generic import DetailView, RedirectView
+from importlib import import_module
 from modeltranslation.settings import AVAILABLE_LANGUAGES
 from purl import URL
 
 from .models import Category, Page
 
 
+logger = logging.getLogger(__name__)
+
+
 class TranslatedSlugMixin(object):
     """
     Returns the object the view is displaying, given a translated slug.
@@ -63,15 +69,30 @@ class SetLanguageView(RedirectView):
         except Http404:
             return '/'
 
-        next = self.request.REQUEST.get('next')
+        next = self.request.REQUEST.get('next', None) or '/'
 
         # Get url slug
         fieldset = dict((k, v) for k, v in match.kwargs.iteritems() if 'slug_' in k)  # NOPEP8
         if fieldset:
+            view_path = match.func.__module__
+            view_name = match.func.func_name
+            root_module = view_path.split('.')[0]
+
+            # Import and define the module containing target view
+            try:
+                globals()[root_module] = import_module(root_module)
+            except ImportError:
+                logger.error(
+                    'Cannot import module %s from matching view.' % (
+                        root_module
+                    )
+                )
+                return next
+
             # Instantiate the view
             #
             # Known issue: this may not work with function based views
-            exec('view_class = %s()' % match.func.func_name)
+            exec('view_class = %s.%s()' % (view_path, view_name))
             if hasattr(view_class, 'model'):  # NOQA
                 obj = view_class.model.objects.get(**fieldset)  # NOQA
                 next = obj.get_absolute_url()
